<?php
/**
 * AMD Telecom
 *
 * DISCLAIMER
 *
 * AMD Telecom grants you a revocable, non-exclusive, non-transferable, limited
 * license to download, install and use the Router Application solely for your commercial
 * purposes strictly in accordance with the terms of this Agreement.
 *
 * @author AMD Telecom <products@amdtelecom.net>
 * @copyright  AMD Telecom SA
 * @license https://go.routee.net
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

class Routee extends Module
{
    private $eventUrl = "https://cmsevents.routee.net/api/event";
    private $dataUrl = "https://cmsevents.routee.net/api/data";
    private $loginUrl = "https://waymore.routee.net/api/authenticate";

    public function __construct()
    {
        $this->name = 'routee';
        $this->tab = 'advertising_marketing';
        $this->version = '3.2.8';
        $this->author = 'Routee';
        $this->need_instance = 0;
        $this->bootstrap = true;

        parent::__construct();

        /*  /OLD DESCRIPTION\
            "Use Routee Platform to create marketing automation. </br>" .
            "Send automated SMS, Emails, Push Notifications, Voice and Viber messages on events of your choice. </br>" .
            "<a target=\"_blank\" href='https://www.routee.net/integrations/prestashop'><strong>Read More</strong></a>"
         */

        $this->displayName = $this->l('Routee');
        $this->description = $this->l('Use Routee Platform to create marketing automation.(SMS, Emails, Viber etc)');

        $this->confirmUninstall = $this->l('Are you sure you want uninstall this module?');

        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_);

        if (!Configuration::get('routee')) {
            $this->warning = $this->l('To use this module you must enter credentials from go.route.net');
        }
    }

    /**
     * On install event
     *
     * @return bool
     */
    public function install()
    {
        if (!parent::install()) {
            return false;
        }

        Db::getInstance()->execute("DELETE FROM `" . _DB_PREFIX_ . "configuration` WHERE name='uuid';");

        return true;
    }

    /**
     * On uninstall event
     *
     * @return bool
     */
    public function uninstall()
    {
        $curlData = array(
            "event" => "Uninstall",
            "uuid" => $this->getUUID(),
            "data" => new stdClass()
        );

        Configuration::deleteByName('routee');
        $this->unsetEvents();
        $this->request($this->eventUrl, $curlData, "POST");
        Db::getInstance()->execute("DELETE FROM `" . _DB_PREFIX_ . "configuration` WHERE name='uuid';");

        return parent::uninstall();
    }

    /**
     * Unregister events
     *
     * @return bool
     */
    public function unsetEvents()
    {
        if (!$this->unregisterHook('actionCartSave')) {
            return false;
        }
        if (!$this->unregisterHook('actionCustomerAccountAdd')) {
            return false;
        }
        if (!$this->unregisterHook('actionCustomerAccountUpdate')) {
            return false;
        }
        if (!$this->unregisterHook('actionValidateCustomerAddressForm')) {
            return false;
        }
        if (!$this->unregisterHook('actionOrderStatusPostUpdate')) {
            return false;
        }
        //if (!$this->unregisterHook('actionOrderStatusUpdate')) {
        //    return false;
        //}
        if (!$this->unregisterHook('actionValidateOrder')) {
            return false;
        }
        if (!$this->unregisterHook('actionPaymentConfirmation')) {
            return false;
        }
        if (!$this->unregisterHook('actionProductAdd')) {
            return false;
        }
        if (!$this->unregisterHook('actionProductUpdate')) {
            return false;
        }
        if (!$this->unregisterHook('actionProductOutOfStock')) {
            return false;
        }
        if (!$this->unregisterHook('actionUpdateQuantity')) {
            return false;
        }
        //if (!$this->unregisterHook('actionProductDelete')) {
        //    return false;
        //}
        if (!$this->unregisterHook('actionSearch')) {
            return false;
        }

        return true;
    }

    /**
     * Register events
     *
     * @return bool
     */
    public function setEvents()
    {
        if (!$this->registerHook('actionCartSave')) {
            return false;
        }
        if (!$this->registerHook('actionCustomerAccountAdd')) {
            return false;
        }
        if (!$this->registerHook('actionCustomerAccountUpdate')) {
            return false;
        }
        if (!$this->registerHook('actionValidateCustomerAddressForm')) {
            return false;
        }
        if (!$this->registerHook('actionOrderStatusPostUpdate')) {
            return false;
        }
        //if (!$this->registerHook('actionOrderStatusUpdate')) {
        //    return false;
        //}
        if (!$this->registerHook('actionValidateOrder')) {
            return false;
        }
        if (!$this->registerHook('actionPaymentConfirmation')) {
            return false;
        }
        if (!$this->registerHook('actionProductAdd')) {
            return false;
        }
        if (!$this->registerHook('actionProductUpdate')) {
            return false;
        }
        if (!$this->registerHook('actionProductOutOfStock')) {
            return false;
        }
        if (!$this->registerHook('actionUpdateQuantity')) {
            return false;
        }
        //if (!$this->registerHook('actionProductDelete')) {
        //    return false;
        //}
        if (!$this->registerHook('actionSearch')) {
            return false;
        }

        return true;
    }

    /**
     * Plugin Configuration Form
     */
    public function getContent()
    {
        $output = null;

        if (Tools::isSubmit('submit' . $this->name)) {
            $username = (string)Tools::getValue('username');
            $password = (string)Tools::getValue('password');

            $domain = Tools::getHttpHost(true) . __PS_BASE_URI__;

            $data = array();
            $data["username"] = $username;
            $data["password"] = $password;
            $data["domain"] = $domain;
            $data["source"] = "PrestaShop";
            $data["type"] = "cms";
            $data["version"] = _PS_VERSION_;
            $data["plugin_version"] = $this->version;

            $db_uuid = Db::getInstance()->executeS(
                "SELECT value FROM `" . _DB_PREFIX_ . "configuration` WHERE name='uuid';"
            );
            $qtyUUID = count($db_uuid);

            if ($qtyUUID == 0) {
                $res = $this->request($this->loginUrl, $data, "POST");

                if ($res["success"]) {
                    if ($this->setEvents()) {
                        $uuid = $res["response"]->uuid;
                        Db::getInstance()->execute(
                            "INSERT INTO `" . _DB_PREFIX_ . "configuration` (name, value) VALUES ('uuid', '$uuid');"
                        );

                        $setDataUrl = $domain . "modules/routee/mass_data.php";
                        $this->request($setDataUrl, array(), "POST");

                        Configuration::updateValue('routee', true);
                        $output .= $this->displayConfirmation($this->l($res["response"]->message));
                   } else {
                       $output .= $this->displayError($this->l(
                           "Routee was unable to register the event hooks due to your shop's configurations."
                        ));
                    }
                } else {
                    $output .= $this->displayError($this->l($res["response"]->message));
          
                }
            } else {
                $output .= $this->displayConfirmation($this->l("You are already authorized."));
            }
        }
        return $output . $this->displayForm();
    }

    public function displayForm()
    {
        // Get default language
        $default_lang = (int)Configuration::get('PS_LANG_DEFAULT');

        // Init Fields form array
        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Settings'),
                ),
                'input' => array(
                    array(
                        'type' => 'title',
                        'text' => 'Before login you must create an account at ' .
                            '<a href="https://go.routee.net" target="_blank">go.routee.net</a>'
                    ),
                    array(
                        'type' => 'username',
                        'label' => $this->l('Username:'),
                        'name' => 'username',
                        'size' => 64,
                        'required' => true
                    ),
                    array(
                        'type' => 'password',
                        'label' => $this->l('Password:'),
                        'name' => 'password',
                        'size' => 64,
                        'required' => true
                    ),
                    array(
                        'type' => 'paragraph',
                        'text' => "Use Routee Platform to create marketing automation. </br>" .
                            "Send automated SMS, Emails, Push Notifications, " .
                            "Voice and Viber messages on events of your choice. </br>" .
                            "<a target=\"_blank\" href='https://www.routee.net/integrations/prestashop'>" .
                            "<strong>Read More</strong></a>"
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Submit'),
                    'class' => 'btn btn-default pull-right'
                )
            )
        );

        $helper = new HelperForm();

        // Module, token and currentIndex
        $helper->module = $this;
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex . '&configure=' . $this->name;

        // Language
        $helper->default_form_language = $default_lang;
        $helper->allow_employee_form_lang = $default_lang;

        // Title and toolbar
        $helper->title = $this->displayName;
        $helper->show_toolbar = true;        // false -> remove toolbar
        $helper->toolbar_scroll = true;      // yes - > Toolbar is always visible on the top of the screen.
        $helper->submit_action = 'submit' . $this->name;
        $helper->toolbar_btn = array(
            'save' =>
                array(
                    'desc' => $this->l('Save'),
                    'href' => AdminController::$currentIndex . '&configure=' . $this->name . '&save' . $this->name .
                        '&token=' . Tools::getAdminTokenLite('AdminModules'),
                ),
            'back' => array(
                'href' => AdminController::$currentIndex . '&token=' . Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('Back to list')
            )
        );

        // Load current value
        //$helper->fields_value['routee'] = Configuration::get('routee');

        return $helper->generateForm(array($fields_form));
    }

    /**
     * Send mass data
     *
     * @return bool
     */
    public function setData()
    {
        $this->setEvents();
        $uuid = $this->getUUID();
        $cartUrl = Tools::getHttpHost(true) . __PS_BASE_URI__ . "cart?action=show";

        $offset = 0;
        $limit = 100;
        $flag = true;
        $delayCounter = 0;
        while ($flag) {
            $customers = Db::getInstance()->executeS(
                "SELECT * FROM " . _DB_PREFIX_ . "customer ORDER BY id_customer LIMIT $limit OFFSET $offset"
            );
            $returnData = array();
            if ((is_array($customers) || is_object($customers))) {
                foreach ($customers as $customer) {
                    $id = $customer["id_customer"];

                    //Set customer format
                    $customerParsed = array(
                        "id" => $customer["id_customer"],
                        "lastname" => $customer["lastname"],
                        "firstname" => $customer["firstname"],
                        "birthday" => $customer["birthday"],
                        "email" => $customer["email"],
                        "company" => $customer["company"],
                        "newsletter" => ($customer["newsletter"] == false) ? false : true,
                        "website" => $customer["website"],
                        "date_add" => $customer["date_add"],
                        "date_upd" => $customer["date_upd"]
                    );
                    $addresses = Db::getInstance()->executeS(
                        "SELECT * FROM " . _DB_PREFIX_ . "address WHERE id_customer='$id'"
                    );
                    $addressesParsed = array();

                    //Set addresses format
                    foreach ($addresses as $address) {
                        $addressParsed = array(
                            "lastname" => $address["lastname"],
                            "firstname" => $address["firstname"],
                            "address1" => $address["address1"],
                            "address2" => $address["address2"],
                            "postcode" => $address["postcode"],
                            "phone" => $address["phone"],
                            "phone_mobile" => $address["phone_mobile"],
                            "city" => $address["city"],
                            "country" => $this->getCountryById($address["id_country"]),
                            "company" => $address["company"]
                        );
                        $addressesParsed[] = $addressParsed;
                    }
                    $carts = Db::getInstance()->executeS(
                        "SELECT * FROM " . _DB_PREFIX_ . "cart WHERE id_customer='$id'"
                    );
                    $respCartProducts = array();
                    $respCartDetails = new stdClass();
                    // Fill cart with products
                    if (is_array($carts) || is_object($carts)) {
                        foreach ($carts as $cart) {
                            $cartId = $cart['id_cart'];
                            $orders = Db::getInstance()->executeS(
                                "SELECT * FROM " . _DB_PREFIX_ . "orders WHERE id_cart='$cartId'"
                            );
                            $cart_products = Db::getInstance()->executeS(
                                "SELECT * FROM " . _DB_PREFIX_ . "cart_product WHERE id_cart='$cartId'"
                            );
                            if (!(is_array($orders) && sizeof($orders) > 0)) {
                                if (is_array($cart_products) && sizeof($cart_products) > 0) {
                                    $respCartDetails->date_add = $cart["date_add"];
                                    $respCartDetails->date_upd = $cart["date_upd"];

                                    foreach ($cart_products as $cart_product) {
                                        $temp_cart_product = array();
                                        $temp_cart_product["product_id"] = $cart_product->id_product;
                                        $temp_cart_product["option_id"] = $cart_product->id_product_attribute;
                                        $temp_cart_product["quantity"] = $cart_product->quantity;
                                        $temp_cart_product["quantity"] = $cart_product->quantity;
                                        $temp_cart_product["date_add"] = $cart_product->date_add;
                                        $respCartProducts[] = $temp_cart_product;
                                    }
                                }
                            }
                        }
                    }

                    $tempReturnObject = new stdClass();
                    $tempReturnObject->customer = $customerParsed;
                    $tempReturnObject->addresses = $addressesParsed;
                    $tempReturnObject->cart = $respCartDetails;
                    $tempReturnObject->cart->cart_products = $respCartProducts;
                    $tempReturnObject->cart->cart_url = $cartUrl;
                    $returnData[] = $tempReturnObject;
                }
            }
            $custObject = new stdClass();
            $custObject->name = "customers";
            $custObject->description = "customers with cart , cart url and products in cart";
            $custObject->object = $returnData;
            $this->sendDbData($custObject, $uuid);

            if (count($customers) < $limit) {
                $flag = false;
            }
            $offset += $limit;
            $delayCounter++;

            if ($delayCounter == 2) {
                $delayCounter = 0;
                sleep(1);
            }
        }

        $offset = 0;
        $limit = 100;
        $flag = true;
        while ($flag) {
            $DBProducts = Db::getInstance()->executeS(
                "SELECT * FROM " . _DB_PREFIX_ . "product ORDER BY id_product LIMIT $limit OFFSET $offset"
            );
            $products = array();

            foreach ($DBProducts as $product) {
                $products[] = $this->getProductData($product["id_product"]);
            }

            $prodObject = new stdClass();
            $prodObject->name = "products";
            $prodObject->description = "products with images {data:{} , image:'url'}";
            $prodObject->object = $products;
            $this->sendDbData($prodObject, $uuid);

            if (count($DBProducts) < $limit) {
                $flag = false;
            }
            $offset += $limit;
            if ($delayCounter == 2) {
                $delayCounter = 0;
                sleep(1);
            }
        }

        $offset = 0;
        $limit = 100;
        $flag = true;
        while ($flag) {
            $DBOrders = Db::getInstance()->executeS(
                "SELECT * FROM " . _DB_PREFIX_ . "orders ORDER BY id_order LIMIT $limit OFFSET $offset"
            );
            $langId = $this->context->language->id;
            $orders = array();

            foreach ($DBOrders as $DBOrder) {
                $order = new Order($DBOrder["id_order"], $langId);
                $customer = new Customer($order->getCustomer()->id);
                $orderStatus = $order->getCurrentOrderState();
                $address = $customer->getAddresses($langId)[0];
                $orderProducts = $order->getProducts();
                $orderProductsParsed = array();

                foreach ($orderProducts as $product) {
                    $temp = array();
                    $temp["product_id"] = $product["product_id"];
                    $temp["option_id"] = $product["product_attribute_id"];
                    $temp["quantity"] = $product["product_quantity"];
                    $orderProductsParsed[] = $temp;
                }
                $orderData = array();
                $orderData["order_id"] = $order->id;
                $orderData["status"] = $orderStatus->name[$langId];
                $orderData["shipping_method"] = $this->getCarrierName(11);
                $orderData["payment_method"] = $order->payment;
                $orderData["total_price"] = $order->total_paid_tax_incl;
                $orderData["customer_id"] = $order->id_customer;
                $orderData["birthday"] = $customer->birthday;
                $orderData["email"] = $customer->email;
                $orderData["newsletter"] = $customer->newsletter;
                $orderData["website"] = $customer->website;
                $orderData["company"] = $address["company"];
                $orderData["lastname"] = $address["lastname"];
                $orderData["firstname"] = $address["firstname"];
                $orderData["address1"] = $address["address1"];
                $orderData["address2"] = $address["address2"];
                $orderData["postcode"] = $address["postcode"];
                $orderData["city"] = $address["city"];
                $orderData["country"] = $address["country"];
                $orderData["phone"] = $address["phone"];
                $orderData["phone_mobile"] = $address["phone_mobile"];
                $orderData["date_add"] = $address["date_add"];
                $orderData["date_upd"] = $address["date_upd"];
                $orderData["order_products"] = $orderProductsParsed;
                $orders[] = $orderData;
            }
            $orderObject = new stdClass();
            $orderObject->name = "orders";
            $orderObject->description = "orders with customer's details";
            $orderObject->object = $orders;
            $this->sendDbData($orderObject, $uuid);

            if (count($DBOrders) < $limit) {
                $flag = false;
            }
            $offset += $limit;
            if ($delayCounter == 2) {
                $delayCounter = 0;
                sleep(1);
            }
        }
    }

    /**
     * Get private properties of class
     *
     * @param $obj
     * @param $name
     * @return mixed
     */
    private function getPrivateValue($obj, $name)
    {
        $r = new ReflectionObject($obj);
        $p = $r->getProperty($name);
        $p->setAccessible(true);
        return $p->getValue($obj);
    }

    /**
     * Get private properties of parent class
     *
     * @param $obj
     * @param $name
     * @return mixed
     */
    private function getParentValue($obj, $name)
    {
        $r = new ReflectionObject($obj);
        $p = $r->getParentClass()->getProperty($name);
        $p->setAccessible(true);
        return $p->getValue($obj);
    }

    /**
     * Get the UUID from DB
     *
     * @return string
     */
    private function getUUID()
    {
        $uuid = Db::getInstance()->executeS("SELECT value FROM `" . _DB_PREFIX_ . "configuration` WHERE name='uuid';");
        return $uuid[0]['value'];
    }

    /**
     * Get a country name
     *
     * @param $id
     * @return mixed
     */
    private function getCountryById($id)
    {
        $langId = $this->context->language->id;
        $countryResult = Db::getInstance()->executeS(
            "SELECT `name` FROM " . _DB_PREFIX_ . "country_lang WHERE id_country='$id' AND id_lang='$langId'"
        );
        return $countryResult[0]['name'];
    }

    /**
     * Get product options
     *
     * @param $product_id
     * @return array
     */
    public function getProductOptions($product_id)
    {
        $finalOptions = array();
        try {
            $product = new Product($product_id);
            $productOptions = $product->getAttributeCombinations();

            $parsedProductOptions = array();
            foreach ($productOptions as $option) {
                $key = $option["id_product_attribute"];
                $name = Tools::strtolower($option["group_name"]);
                $value = $option["attribute_name"];

                $parsedProductOptions[$key]["option_id"] = $key;
                $parsedProductOptions[$key][$name] = $value;
                $parsedProductOptions[$key]["stock_quantity"] = $option["quantity"];
                $parsedProductOptions[$key]["available_date"] = $option["available_date"];
                $parsedProductOptions[$key]["price_impact"] = $option["price"] * (1 + ($product->getTaxesRate() / 100));
            }

            foreach ($parsedProductOptions as $option) {
                $finalOptions[] = $option;
            }
        } catch (PrestaShopDatabaseException $e) {
        } catch (PrestaShopException $e) {
        }

        return $finalOptions;
    }

    /**
     * Get image link of a product
     *
     * @param $id_product
     * @return string
     */
    private function getImgUrl($id_product)
    {
        $imagePath = null;
        try {
            $product = new Product((int)$id_product, false, Context::getContext()->language->id);
            $image = Image::getCover($id_product);
            $link = new Link;

            if (method_exists('ImageType','getFormattedName')) {
                $imageType = ImageType::getFormattedName('home');
            } else {
                $imageType = ImageType::getFormatedName('home');
            }
            
            $imageName = isset($product->link_rewrite) ? $product->link_rewrite : $product->name;
            $imagePath = $link->getImageLink($imageName, (int)$image['id_image'], $imageType);
        } catch (PrestaShopDatabaseException $e) {
        } catch (PrestaShopException $e) {
        }

        return $imagePath;
    }

    /**
     * Get the link of a product
     *
     * @param $productId
     * @return string
     */
    private function getProductUrl($productId)
    {
        $productUrl = null;
        try {
            $link = new Link();
            $product = new Product($productId);
            $productUrl = $link->getProductLink($product);
        } catch (PrestaShopDatabaseException $e) {
        } catch (PrestaShopException $e) {
        }


        return $productUrl;
    }

    /**
     * Get the price of a product without any discounts
     *
     * @param $productId
     * @return float
     */
    private function getProductOriginalPrice($productId)
    {
        $price = null;
        try {
            $product = new Product($productId);
            $price = $product->getPriceWithoutReduct(false, null, 2);
        } catch (PrestaShopDatabaseException $e) {
        } catch (PrestaShopException $e) {
        }

        return $price;
    }

    /**
     * Get carrier of an order
     *
     * @param $id_order
     * @return mixed
     */
    private function getCarrierName($id_order)
    {
        $query = 'SELECT cl.`name` as `carrier_name` FROM `' .
            _DB_PREFIX_ . 'order_carrier` oc LEFT JOIN `' .
            _DB_PREFIX_ . 'carrier` cl ON (oc.`id_carrier` = cl.`id_carrier`) WHERE oc.`id_order` = ' . (int)$id_order;
        $DBCarrier = Db::getInstance()->executeS($query);
        return $DBCarrier[0]["carrier_name"];
    }

    /**
     * Get product custom data
     *
     * @param $productId
     * @param null | $productObject
     * @return array
     */
    private function getProductData($productId, $productObject = null)
    {
        if ($productObject == null) {
            $productObject = new stdClass();

            try {
                $tempProductObject = new Product($productId);
                $productObject->name = $tempProductObject->name;
                $productObject->description = $tempProductObject->name;
                $productObject->description_short = $tempProductObject->name;
                $productObject->quantity = Product::getQuantity($productId);
            } catch (PrestaShopDatabaseException $e) {
            } catch (PrestaShopException $e) {
            }
        }

        $langId = $this->context->language->id;
        //$currencySign = $this->context->currency->sign;
        $productUrl = $this->getProductUrl($productId);
        $productImage = $this->getImgUrl($productId);
        $price = $this->getProductOriginalPrice($productId);
        //$staticPrice = Product::getPriceStatic($productId);
        $tempCategories = Product::getProductCategoriesFull($productId);
        $categories = array();

        foreach ($tempCategories as $category) {
            $categories[] = $category["name"];
        }

        $specificPrices = SpecificPrice::getByProductId($productId);
        $discounts = array();

        foreach ($specificPrices as $specificPrice) {
            $discountPrice = "";
            $reductionType = Tools::strtolower($specificPrice["reduction_type"]);

            if ($reductionType == "percentage") {
                $discountPrice = $price - ($price * $specificPrice["reduction"]);
            } elseif ($reductionType == "amount") {
                $reduction = $specificPrice["reduction"];

                if ($specificPrice["reduction_tax"] == false) {
                    //TODO: price with taxless reduction
                    continue;
                }
                $discountPrice = $price - $reduction;
            }

            //TODO: check date from-to and set staticPrice

            $temp_discounts = array(
                "price" => round($discountPrice, 2),
                "date_start" => $specificPrice["from"],
                "date_end" => $specificPrice["to"],
                "quantity" => "",
            );
            $discounts[] = $temp_discounts;
        }

        $data = array(
            "product_id" => $productId,
            "name" => $productObject->name[$langId],
            "description" => $productObject->description[$langId],
            "short_description" => $productObject->description_short[$langId],
            "categories" => $categories,
            "stock_quantity" => $productObject->quantity,
            "price" => $price,
            "discounts" => $discounts,
            "image_link" => $productImage,
            "product_link" => $productUrl,
            "product_options" => $this->getProductOptions($productId)
        );

        return $data;
    }

    /**
     * Send mass data
     *
     * @param $data
     * @param $uuid
     */
    private function sendDbData($data, $uuid)
    {
        $curlData = array(
            "data" => array(
                $data
            ),
            "uuid" => $uuid,
        );

        $this->request($this->dataUrl, $curlData, "POST");
    }

    /**
     * CURL Request
     *
     * @param $url
     * @param $data
     * @param $method
     * @return array
     */
    private function request($url, $data, $method)
    {
        $curl = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 6,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => $method,
            CURLOPT_HTTPHEADER => array("content-type: application/json"),
            CURLOPT_POSTFIELDS => json_encode($data)
        ));

        $curl_response = curl_exec($curl);
        $error = curl_error($curl);
        $code = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        curl_close($curl);

        $flag = false;
        if (!$error && $code == 200) {
            $flag = true;
        }

        $response = array();
        $response["success"] = $flag;
        $response["data"] = array();
        $response["data"]["code"] = $code;
        $response["data"]["error"] = $error;
        $response["response"] = new stdClass();

        if ($curl_response == null) {
             $response["response"]->message = 'Request timed out. Please find a better connection or try again later.';
        } else {
            $response["response"] = json_decode($curl_response);
        }

        return $response;
    }

    /* ============================================ */
    /*                HOOK FUNCTIONS                */
    /* ============================================ */

    public function hookActionCartSave(array $params)
    {
        if (isset($params['cart']->id) && isset($params['cart']->id_customer)) {
            $id = $params['cart']->id;
            $customerId = $params['cart']->id_customer;

            if ($customerId > 0) {
                $cartUrl = Tools::getHttpHost(true) . __PS_BASE_URI__ . "cart?action=show";
                $data = new stdClass();
                $orders = Db::getInstance()->executeS(
                    "SELECT * FROM " . _DB_PREFIX_ . "orders WHERE id_cart='$id'"
                );

                if (!(is_array($orders) && count($orders) > 0)) {
                    $data->customer_id = $customerId;
                    $data->cart_products = array();
                    $data->cart_url = $cartUrl;

                    $cart_products = Db::getInstance()->executeS(
                        "SELECT * FROM " . _DB_PREFIX_ . "cart_product WHERE id_cart='$id'"
                    );

                    if (is_array($cart_products) && count($cart_products) > 0) {
                        foreach ($cart_products as $cart_product) {
                            $cartPro = new stdClass();
                            $cartPro->product_id = $cart_product['id_product'];
                            $cartPro->quantity = $cart_product['quantity'];
                            $cartPro->option_id = $cart_product['id_product_attribute'];
                            $data->cart_products[] = $cartPro;
                        }
                    }
                }

                $curlData = array(
                    "event" => "CartUpdate",
                    "data" => $data,
                    "uuid" => $this->getUUID()
                );

                $this->request($this->eventUrl, $curlData, "POST");
            }
        }
    }

    public function hookActionCustomerAccountAdd(array $params)
    {
        $customerParsed = array(
            "id" => $params["newCustomer"]->id,
            "lastname" => $params["newCustomer"]->lastname,
            "firstname" => $params["newCustomer"]->firstname,
            "birthday" => $params["newCustomer"]->birthday,
            "email" => $params["newCustomer"]->email,
            "company" => $params["newCustomer"]->company,
            "newsletter" => ($params["newCustomer"]->newsletter == false) ? false : true,
            "website" => $params["newCustomer"]->website
        );

        $curlData = array(
            "event" => "CustomerAdd",
            "data" => array(
                "customer" => $customerParsed
            ),
            "uuid" => $this->getUUID()
        );

        $this->request($this->eventUrl, $curlData, "POST");
    }

    public function hookActionCustomerAccountUpdate(array $params)
    {
        $id = $params["customer"]->id;
        $addresses = Db::getInstance()->executeS("SELECT * FROM " . _DB_PREFIX_ . "address WHERE id_customer='$id'");

        $customerParsed = array(
            "id" => $params["customer"]->id,
            "lastname" => $params["customer"]->lastname,
            "firstname" => $params["customer"]->firstname,
            "birthday" => $params["customer"]->birthday,
            "email" => $params["customer"]->email,
            "company" => $params["customer"]->company,
            "newsletter" => ($params["customer"]->newsletter == false) ? false : true,
            "website" => $params["customer"]->website
        );

        $addressesParsed = array();

        foreach ($addresses as $address) {
            $addressParsed = array(
                "lastname" => $address["lastname"],
                "firstname" => $address["firstname"],
                "address1" => $address["address1"],
                "address2" => $address["address2"],
                "postcode" => $address["postcode"],
                "phone" => $address["phone"],
                "phone_mobile" => $address["phone_mobile"],
                "city" => $address["city"],
                "country" => $this->getCountryById($address["id_country"]),
                "company" => $address["company"]
            );

            $addressesParsed[] = $addressParsed;
        }

        $curlData = array(
            "event" => "CustomerUpdate",
            "data" => array(
                "customer" => $customerParsed,
                "addresses" => $addressesParsed
            ),
            "uuid" => $this->getUUID()
        );

        $this->request($this->eventUrl, $curlData, "POST");
    }

    public function hookActionValidateCustomerAddressForm(array $params)
    {
        $id = $params['cart']->id_customer;
        $customer = Db::getInstance()->executeS("SELECT * FROM " . _DB_PREFIX_ . "customer WHERE id_customer='$id'");
        $addresses = Db::getInstance()->executeS("SELECT * FROM " . _DB_PREFIX_ . "address WHERE id_customer='$id'");


        $formFields = $this->getPrivateValue($params["form"], "formFields");
        $firstname = $this->getParentValue($formFields["firstname"], "value");
        $lastname = $this->getParentValue($formFields["lastname"], "value");
        $company = $this->getParentValue($formFields["company"], "value");
        $address1 = $this->getParentValue($formFields["address1"], "value");
        $address2 = $this->getParentValue($formFields["address2"], "value");
        $postcode = $this->getParentValue($formFields["postcode"], "value");
        $city = $this->getParentValue($formFields["city"], "value");
        $phone = $this->getParentValue($formFields["phone"], "value");
        $countryId = $this->getParentValue($formFields["id_country"], "value");
        $country = $this->getCountryById($countryId);
        $mobile = "";
        try {
            $mobile = $this->getParentValue($formFields["phone"], "value");
        } catch (Exception $e) {
        }

        (isset($mobile)) ? $phone_mobile = $mobile : $phone_mobile = "";

        $customerParsed = array(
            "id" => $customer[0]["id_customer"],
            "lastname" => $customer[0]["lastname"],
            "firstname" => $customer[0]["firstname"],
            "birthday" => $customer[0]["birthday"],
            "email" => $customer[0]["email"],
            "company" => $customer[0]["company"],
            "newsletter" => ($customer[0]["newsletter"] == false) ? false : true,
            "website" => $customer[0]["website"]
        );

        $addressesParsed = array();

        foreach ($addresses as $address) {
            $addressParsed = array(
                "lastname" => $address["lastname"],
                "firstname" => $address["firstname"],
                "address1" => $address["address1"],
                "address2" => $address["address2"],
                "postcode" => $address["postcode"],
                "phone" => $address["phone"],
                "phone_mobile" => $address["phone_mobile"],
                "city" => $address["city"],
                "country" => $this->getCountryById($address["id_country"]),
                "company" => $address["company"]
            );

            $addressesParsed[] = $addressParsed;
        }

        $addressesParsed[] = array(
            "firstname" => $firstname,
            "lastname" => $lastname,
            "address1" => $address1,
            "address2" => $address2,
            "postcode" => $postcode,
            "phone" => $phone,
            "phone_mobile" => $phone_mobile,
            "city" => $city,
            "country" => $country,
            "company" => $company
        );

        $curlData = array(
            "event" => "CustomerUpdate",
            "data" => array(
                "customer" => $customerParsed,
                "addresses" => $addressesParsed
            ),
            "uuid" => $this->getUUID()
        );

        $this->request($this->eventUrl, $curlData, "POST");
    }

    public function hookActionOrderStatusPostUpdate(array $params)
    {
        $data = array(
            "customer_id" => $params["cart"]->id_customer,
            "order_id" => $params["id_order"],
            "order_status" => $params["newOrderStatus"]->name
        );

        $curlData = array(
            "event" => "OrderStatusUpdate",
            "data" => $data,
            "uuid" => $this->getUUID()
        );
        $this->request($this->eventUrl, $curlData, "POST");
    }

    public function hookActionOrderStatusUpdate(array $params)
    {
    }

    public function hookActionValidateOrder(array $params)
    {
        $addresses = Db::getInstance()->executeS(
            "SELECT * FROM " . _DB_PREFIX_ . "address WHERE id_customer='" . $params["customer"]->id . "'"
        );
        //$currencySign = $this->context->currency->sign;

        $order_details = array(
            "order_id" => $params["order"]->id,
            "order_status" => $params["orderStatus"]->name,
            "shipping_method" => $this->getCarrierName($params["order"]->id),
            "payment_method" => $params["order"]->payment,
            "total_price" => $params["order"]->total_paid,
            "customer_id" => $params["customer"]->id,
            "lastname" => $params["customer"]->lastname,
            "firstname" => $params["customer"]->firstname,
            "birthday" => $params["customer"]->birthday,
            "email" => $params["customer"]->email,
            "company" => $params["customer"]->company,
            "newsletter" => ($params["customer"]->newsletter == false) ? false : true,
            "website" => $params["customer"]->website,
            "address1" => $addresses[0]["address1"],
            "address2" => $addresses[0]["address2"],
            "postcode" => $addresses[0]["postcode"],
            "phone" => $addresses[0]["phone"],
            "phone_mobile" => $addresses[0]["phone_mobile"],
            "city" => $addresses[0]["city"],
            "country" => $this->getCountryById($addresses[0]["id_country"])
        );

        $order_products = array();

        foreach ($params["order"]->product_list as $product) {
            $temp_product = array();
            $temp_product["product_id"] = $product["id_product"];
            $temp_product["quantity"] = $product["cart_quantity"];
            $temp_product["option_id"] = $product["id_product_attribute"];
            $order_products[] = $temp_product;
        }

        $data = array(
            "customer_id" => $params["order"]->id_customer,
            "order_details" => $order_details,
            "order_products" => $order_products
        );

        $curlData = array(
            "event" => "OrderAdd",
            "data" => $data,
            "uuid" => $this->getUUID()
        );
        $this->request($this->eventUrl, $curlData, "POST");
    }

    public function hookActionPaymentConfirmation(array $params)
    {
        $data = array(
            "order_id" => $params["id_order"],
            "customer_id" => $params["cart"]->id_customer
        );

        $curlData = array(
            "event" => "OrderPaymentConfirmed",
            "data" => $data,
            "uuid" => $this->getUUID()
        );
        $this->request($this->eventUrl, $curlData, "POST");
    }

    public function hookActionProductAdd(array $params)
    {
        $data = $this->getProductData($params["id_product"], $params["product"]);

        $curlData = array(
            "data" => $data,
            "event" => "ProductAdd",
            "uuid" => $this->getUUID()
        );

        $this->request($this->eventUrl, $curlData, "POST");
    }

    public function hookActionProductUpdate(array $params)
    {
        $data = $this->getProductData($params["id_product"], $params["product"]);

        $curlData = array(
            "data" => $data,
            "event" => "ProductUpdate",
            "uuid" => $this->getUUID()
        );

        $this->request($this->eventUrl, $curlData, "POST");
    }

    public function hookActionProductOutOfStock(array $params)
    {
        $id = $params["product"]->id;
        if (is_numeric($id)) {
            $curlData = array(
                "event" => "ProductOutOfStock",
                "data" => array(
                    "product_id" => $id
                ),
                "uuid" => $this->getUUID());
            $this->request($this->eventUrl, $curlData, "POST");
        }
    }

    public function hookActionUpdateQuantity(array $params)
    {
        $productId = $params["id_product"];
        $quantity = $params["quantity"];
        $optionId = $params["id_product_attribute"];

        $curlData = array(
            "event" => "ProductStockUpdate",
            "data" => array(
                "product_id" => $productId,
                "quantity" => $quantity,
                "option_id" => $optionId
            ),
            "uuid" => $this->getUUID());
        $this->request($this->eventUrl, $curlData, "POST");
    }

    public function hookActionProductDelete(array $params)
    {
    }

    public function hookActionSearch(array $params)
    {
        $params["searched_query"];

        $data = array(
            "search_string" => $params["searched_query"],
            "customer_id" => $params["cart"]->id_customer,
            "guest_id" => $params["cart"]->id_guest,
        );

        $curlData = array(
            "event" => "Search",
            "data" => $data,
            "uuid" => $this->getUUID()
        );
        $this->request($this->eventUrl, $curlData, "POST");
    }
}
